import { ManageListener } from "gestion-listener"
import { Client, DMChannel, Message, NewsChannel, TextChannel } from "discord.js"

import {show} from "./show"
const path = require("path");

import { openSync } from "fs"

const config: {'discord': {'token': string}} = require('../config.json')

// setup Discord
const monClient = new Client();
let channel_on: Array<TextChannel | NewsChannel | DMChannel> = [];

// setup gestion listener
let mesMl: {ml: ManageListener, channel: TextChannel | NewsChannel | DMChannel}[] = [];
// const ml = new ManageListener(path.join(__dirname, '../save/tmp.db'));

const fun_rss = async (message: Message, args: string[], ml: ManageListener) : Promise<void> => {
  switch(args[0]) {
    case 'start':
      channel_on.push(message.channel);
      await ml.startAll();
      console.log(`Demarrage du service sur ${message.channel.toString()}`);
      message.channel.send('Le feed va bientôt apparaitre 😉');
      break;
    case 'stop':
      channel_on.splice(channel_on.indexOf(message.channel));
      await ml.stopAll();
      console.log(`Arret du service sur #${message.channel.toString()}`);
      message.channel.send('Le feed a bien été retiré 🤐');
      break;
    case 'add':
      if(args[1]) {
        ml.registerListener({
          address: args[1]
        });
        message.channel.send(`Ajout du service correspondant au lien : ${args[1]} 👌`);
      }
      else  message.channel.send(`L'argument doit etre un lien vers un flux rss valide`)
      break;
    case 'help':
    default:
      // console.log(show.show_rss_help());
      message.channel.send(show.show_rss_help());
      break;
  }
}

const init_ml = async (message: Message) : Promise<ManageListener> => {
  const new_path = path.join(__dirname, `../save/${message.channel.id}`);
  openSync(new_path, 'w');
  const new_ml = new ManageListener(new_path);
  mesMl.push({
    ml: new_ml,
    channel: message.channel
  });
  
  new_ml.on("newEntries", async (obj) => {
    if (obj) {
      obj.map((item: any) => `${item.link}\n\t${item.title} :: Le ${(new Date()).toLocaleTimeString()}\n`)
        .forEach((item: string) => { 
            // console.log(`${item}\n`);
            message.channel.send(`${item}\n`);
          }
        )
    }
    await new_ml.save();
  });

  await new_ml.load();

  return new_ml;
}

monClient.on('message', async (message) => {
  const splited_message = message.content.split(' ');
  if(splited_message[0] === "C2") {
    const index_ml = mesMl.map(i => i.channel).indexOf(message.channel);
    let ml: ManageListener;
    if(!mesMl.map(i => i.channel).includes(message.channel))
      ml = await init_ml(message);
    else
      ml = mesMl[index_ml].ml;
    switch(splited_message[1]) {
      case 'rss':
        await fun_rss(message, splited_message.slice(2), ml);
        break;
      case 'help':
      default:
        // console.log(show.show_help());
        message.channel.send(show.show_help())
        break;
    }
  }
});
monClient.login(config.discord.token);

console.log("Let's goooo");
